@@ -83,22 +83,24 @@ recode_addChildCodes <- function(input,
                                  silent = rock::opts$get('silent')) {
 
   return(
-    generic_recoding(
-      input = input,
-      codes = codes,
-      childCodes = childCodes,
-      filter = filter,
-      func = changeSource_addChildCodes,
-      output = output,
-      filenameRegex = filenameRegex,
-      outputPrefix = outputPrefix,
-      outputSuffix = outputSuffix,
-      decisionLabel = decisionLabel,
-      justification = justification,
-      justificationFile = justificationFile,
-      preventOverwriting = preventOverwriting,
-      encoding = encoding,
-      silent = silent
+    invisible(
+      generic_recoding(
+        input = input,
+        codes = codes,
+        childCodes = childCodes,
+        filter = filter,
+        func = changeSource_addChildCodes,
+        output = output,
+        filenameRegex = filenameRegex,
+        outputPrefix = outputPrefix,
+        outputSuffix = outputSuffix,
+        decisionLabel = decisionLabel,
+        justification = justification,
+        justificationFile = justificationFile,
+        preventOverwriting = preventOverwriting,
+        encoding = encoding,
+        silent = silent
+      )
     )
   );
 
